/* SPDX-License-Identifier: GPL-2.0+ OR BSD-3-Clause */
/*
 * Copyright (C) 2020-2022, STMicroelectronics - All Rights Reserved
 */

#if STM32MP13
#include "stm32mp13-resets.h"
#endif
#if STM32MP15
#include "stm32mp15-resets.h"
#endif
