/* SPDX-License-Identifier: GPL-2.0+ OR BSD-3-Clause */
/*
 * Copyright (C) STMicroelectronics 2018-2022 - All Rights Reserved
 * Author: Gabriel Fernandez <gabriel.fernandez@st.com> for STMicroelectronics.
 */

#if STM32MP13
#include "stm32mp13-clks.h"
#endif
#if STM32MP15
#include "stm32mp15-clks.h"
#endif
