/*
 * Copyright (c) 2022, ARM Limited and Contributors. All rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

#ifndef PLAT_HELPERS_H
#define PLAT_HELPERS_H

unsigned int plat_mediatek_calc_core_pos(u_register_t mpidr);

#endif /* PLAT_HELPERS_H */
