/*
 * Copyright (c) 2016-2022, ARM Limited and Contributors. All rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

.section .bl2_image
.incbin BL2_BIN_PATH

.section .dtb_image
.incbin DTB_BIN_PATH
