/*
 * Copyright (c) 2022, MediaTek Inc. All rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

#ifndef MT_MSDC_PRIV_H
#define MT_MSDC_PRIV_H

#define MSDC_CQHCI_CFG 0x808
#define MSDC_CQHCI_CRYPTO_ENABLE BIT(1)

#endif
