/*
 * Copyright (c) 2015-2022, STMicroelectronics - All Rights Reserved
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

#if STM32MP13
#include "stm32mp13_rcc.h"
#endif
#if STM32MP15
#include "stm32mp15_rcc.h"
#endif
