/*
 * Copyright (c) 2022, ARM Limited and Contributors. All rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

#include <common/debug.h>
#include <emi_mpu.h>

void set_emi_mpu_regions(void)
{
	/* TODO: set emi mpu region */
	INFO("%s, emi mpu is not setting currently\n", __func__);
}
