/*
 * Copyright (c) 2022, Intel Corporation. All rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

#ifndef __S10_MMC_H__
#define __S10_MMC_H__

void s10_mmc_init(void);

#endif /* S10_MMC_H */
