/*
 * Copyright (c) 2022, MediaTek Inc. All rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

#ifndef MT_AUDIO_PRIVATE_H
#define MT_AUDIO_PRIVATE_H

#include <platform_def.h>

#define AFE_SE_SECURE_CON		 (AUDIO_BASE + 0x17a8)
#define AFE_SECURE_SIDEBAND0		 (AUDIO_BASE + 0x1908)
#define AFE_SECURE_SIDEBAND1		 (AUDIO_BASE + 0x190c)
#define AFE_SECURE_SIDEBAND2		 (AUDIO_BASE + 0x1910)
#define AFE_SECURE_SIDEBAND3		 (AUDIO_BASE + 0x1914)

#define SPM_PWR_STATUS_AUDIO_BIT	 (6)

#endif /* MT_AUDIO_PRIVATE_H */
