/*
 * Copyright (c) 2022, ARM Limited and Contributors. All rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

#include <sunxi_private.h>

const struct psci_cpu_idle_state sunxi_idle_states[] = {
	{}
};
