/*
 * Copyright (c) 2022, ARM Limited and Contributors. All rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

#ifndef WA_CVE_2022_23960_H
#define WA_CVE_2022_23960_H

int check_smccc_arch_wa3_applies(void);

#endif /* WA_CVE_2022_23960_H */
