/*
 * Copyright (c) 2022, Fujitsu Limited and Contributors. All rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

#ifndef A64FX_H
#define A64FX_H

#include <lib/utils_def.h>

/* A64FX midr for revision 0 */
#define A64FX_MIDR					U(0x461f0010)

#endif /* A64FX_H */
