/*
 * Copyright (c) 2021-2022, STMicroelectronics - All Rights Reserved
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

#ifndef PLAT_DEF_FIP_UUID_H
#define PLAT_DEF_FIP_UUID_H

#define UUID_STM32MP_CONFIG_CERT \
	{{0x50, 0x1d, 0x8d, 0xd2}, {0x8b, 0xce}, {0x49, 0xa5}, 0x84, 0xeb, \
	 {0x55, 0x9a, 0x9f, 0x2e, 0xae, 0xaf} }
#endif /* PLAT_DEF_FIP_UUID_H */

