/*
 * Copyright (c) 2022, MediaTek Inc. All rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

#ifndef MT_MSDC_H
#define MT_MSDC_H

#include <mt_msdc_priv.h>

uint64_t msdc_smc_dispatcher(uint64_t arg0, uint64_t arg1,
			     uint64_t arg2, uint64_t arg3);

#endif
